@@ -82,6 +82,19 @@ def split_body_parts(num_parts,body_parts):
         parts_dict[part]['content'] = codecs.unicode_escape_decode(MDRenderer().render(tokens[ptt[1]+1:pa[0]], mdit.options, env))[0]
         parts_dict[part]['answer']['content'] = codecs.unicode_escape_decode(MDRenderer().render(tokens[pa[1]+1:], mdit.options, env))[0]
 
+        # Remove parts from body_parts
+        body_parts.pop(part)
+
+    # Deal with other headings: pl-submission-panel and pl-answer-panel
+
+    for key in body_parts.keys():
+        if key in ['pl-submission-panel','pl-answer-panel']:
+            # Set up tokens by parsing the md file
+            tokens = mdit.parse(body_parts[key], env)
+
+            ptt = [i for i,j in enumerate(tokens) if j.tag=='h2']
+            parts_dict[key] = codecs.unicode_escape_decode(MDRenderer().render(tokens[ptt[-1]+1:], mdit.options, env))[0]
+
     return defdict_to_dict(parts_dict,{})
 
 def read_md_problem(filepath):
@@ -158,7 +171,7 @@ def read_md_problem(filepath):
 
     for k,v in blocks.items():
 
-        rendered_part = codecs.unicode_escape_decode(MDRenderer().render(tokens[v[0]:v[1]], mdit.options, env))[0]
+        rendered_part = MDRenderer().render(tokens[v[0]:v[1]], mdit.options, env)
 
         if k == 'title':
             body_parts['title'] = rendered_part
@@ -175,6 +188,12 @@ def read_md_problem(filepath):
         elif 'Comments' in rendered_part:
             body_parts['Comments'] = rendered_part
 
+        elif 'pl-submission-panel' in rendered_part:
+            body_parts['pl-submission-panel'] = rendered_part
+
+        elif 'pl-answer-panel' in rendered_part:
+            body_parts['pl-answer-panel'] = rendered_part
+
         else:
             part_counter +=1
             body_parts['part{0}'.format(part_counter)] = rendered_part
@@ -230,15 +249,25 @@ def write_server_py(output_path,parsed_question):
         parsed_question ([type]): [description]
     """
     
+    server_dict = parsed_question['header']['server']
+    
+    server_file = f""""""
+    
+    server_file += server_dict.pop('imports',None) + '\n'
+    
     try:
-        imp, func = parsed_question['header']['server'].split('# Start problem code')
+        for function, code in server_dict.items():
+            indented_code = code.replace('\n','\n    ')
+            if code:
+                server_file += f"def {function}(data):\n    {indented_code}\n"
     except:
         raise
 
-    func = func.replace('read_csv("','read_csv(data["options"]["client_files_course_path"]+"/')
-    func = func.replace('\n','\n    ')
+    # Deal with path differences when using PL
+    server_file = server_file.replace('read_csv("','read_csv(data["options"]["client_files_course_path"]+"/')
 
-    (output_path / "server.py").write_text(imp+'def generate(data):'+func,encoding='utf8')
+    # Write server.py
+    (output_path / "server.py").write_text(server_file,encoding='utf8')
 
 def process_multiple_choice(part_name,parsed_question, data_dict):
     """Processes markdown format multiple-choice questions and returns PL HTML
@@ -347,6 +376,34 @@ def replace_tags(string):
     """
     return string.replace('|@','{{').replace('@|','}}')
 
+def remove_correct_answers(data2_dict):
+    """Magical recursive function that removes particular keys from a nested dictionary: https://stackoverflow.com/a/29652561/2217577
+
+    Args:
+        data2_dict (dict): Dictionary (nested) from which to remove key:value
+
+    Returns:
+        data2_dict (dict): Dictionary with the offending keys removed
+    """
+
+    # This was adapted from this SO: https://stackoverflow.com/a/29652561/2217577
+    def gen_dict_extract(key_to_remove, dict_object):
+        if hasattr(dict_object,'items'):
+            for k, v in list(dict_object.items()):
+                if key_to_remove in k:
+                    dict_object.pop(k,None)
+                if isinstance(v, dict):
+                    for result in gen_dict_extract(key_to_remove, v):
+                        yield result
+                elif isinstance(v, list):
+                    for d in v:
+                        for result in gen_dict_extract(key_to_remove, d):
+                            yield result
+
+    list(gen_dict_extract('correct',data2_dict))
+
+    return data2_dict
+
 def process_attribution(source):
     """Takes in a string and returns the HTML for the attribution
 
@@ -358,45 +415,132 @@ def process_attribution(source):
     """
     try:
         if 'openstax-physics-vol1' in source:
-            attribution_text = "![Image representing the Creative Commons 4.0 BY license.](https://i.creativecommons.org/l/by/4.0/88x31.png) Problem is from the [OpenStax University Physics Volume 1](https://openstax.org/details/books/university-physics-volume-1) textbook, licensed under the [CC-BY 4.0 license](https://creativecommons.org/licenses/by/4.0/)."
+            attribution_text = "![Image representing the Creative Commons 4.0 BY license.](https://raw.githubusercontent.com/firasm/bits/master/by.png) Problem is from the [OpenStax University Physics Volume 1](https://openstax.org/details/books/university-physics-volume-1) textbook, licensed under the [CC-BY 4.0 license](https://creativecommons.org/licenses/by/4.0/)."
 
         elif 'openstax-physics-vol2' in source:
-            attribution_text = "![Image representing the Creative Commons 4.0 BY license.](https://i.creativecommons.org/l/by/4.0/88x31.png) Problem is from the [OpenStax University Physics Volume 2](https://openstax.org/details/books/university-physics-volume-2) textbook, licensed under the [CC-BY 4.0 license](https://creativecommons.org/licenses/by/4.0/)."
+            attribution_text = "![Image representing the Creative Commons 4.0 BY license.](https://raw.githubusercontent.com/firasm/bits/master/by.png) Problem is from the [OpenStax University Physics Volume 2](https://openstax.org/details/books/university-physics-volume-2) textbook, licensed under the [CC-BY 4.0 license](https://creativecommons.org/licenses/by/4.0/)."
 
         elif 'ubc-mech2' in source:
             raise NotImplementedError
 
         elif 'standard' in source:
-            attribution_text = "![Image representing the Creative Commons 4.0 BY-NC-SA license.](https://mirrors.creativecommons.org/presskit/buttons/88x31/png/by-nc-sa.png) Problem is licensed under the [CC-BY-NC-SA 4.0 license](https://creativecommons.org/licenses/by-nc-sa/4.0/)."
+            attribution_text = "![The Creative Commons 4.0 license requiring attribution-BY, non-commercial-NC, and share-alike-SA license.](https://raw.githubusercontent.com/firasm/bits/master/by-nc-sa.png) Problem is licensed under the [CC-BY-NC-SA 4.0 license](https://creativecommons.org/licenses/by-nc-sa/4.0/)."
     
         return attribution_text
     
     except TypeError:
         print("You probably need to update the template, the 'attribution' key seems to be missing.")
+        
+def process_question_md(source_filepath, output_path = None, instructor = False):
+    
+    try:
+        pathlib.Path(source_filepath)
+    except:
+        print(f"{source_filepath} - File does not exist.")
+        raise
+        
+    if output_path is None:
+        if instructor: 
+            # Set the output path (hard-coded)
+            output_path = pathlib.Path(source_filepath.replace('source','output/instructor'))
+        else:
+            # Set the output path (hard-coded)
+            output_path = pathlib.Path(source_filepath.replace('source','output/public'))
+    else:
+        ## TODO: Make this a bit more robust
+        output_path = pathlib.Path(output_path)
+        print(f"Warning: This feature (specifying your own directory {output_path}) is not tested!")
+    
+    # deal with multi-line strings in YAML Dump
+    ## Code copied from here: https://stackoverflow.com/a/33300001/2217577
 
-def process_question(input_file, output_path):
+    def str_presenter(dumper, data2):
+        if len(data2.splitlines()) > 1:  # check for multiline string
+            return dumper.represent_scalar('tag:yaml.org,2002:str', data2, style='|')
+        return dumper.represent_scalar('tag:yaml.org,2002:str', data2)
+
+    yaml.add_representer(str, str_presenter)
+
+    parsed_q = read_md_problem(source_filepath)
+
+    header = parsed_q['header']
+    body_parts = parsed_q['body_parts']
+    
+    # Run the python code
+    ## TODO: Is there a better way to do this?
+    exec(parsed_q['header']['server']['imports'].replace('import prairielearn as pl','from . import prairielearn as pl'),globals() )
+    exec(parsed_q['header']['server']['generate'].split('# Update the data object with a new dict')[0],globals() )     
+
+    # Remove the solutions from the server section
+    if instructor is True:
+        header.pop('server',None)
+
+        # Remove python solution from the public view
+        header.pop('server',None)
+
+        # Remove correct answers from the data2 dict 
+        data2_sanitized = defdict_to_dict(data2,{})
+        data2_sanitized = remove_correct_answers(data2_sanitized)
+
+        # Update the YAML header to add substitutions 
+        header.update({'substitutions': data2_sanitized})
+
+        # Update the YAML header to add substitutions, unsort it, and process for file
+        header_yml = yaml.dump(header,sort_keys=False,default_flow_style=False)
+        
+        # Write the YAML to a file
+        output_path.parent.mkdir(parents=True, exist_ok=True)
+        output_path.write_text('---\n' + header_yml + '---\n' + dict_to_md(body_parts,remove_keys=['pl-submission-panel','pl-answer-panel']) +
+                        '\n## Attribution\n\n' + process_attribution(header.get('attribution')) ,encoding='utf8')
+        
+    else:
+        # Update the YAML header to add substitutions 
+        header.update({'substitutions': defdict_to_dict(data2,{})})
+
+        # Update the YAML header to add substitutions, unsort it, and process for file
+        header_yml = yaml.dump(header,sort_keys=False,default_flow_style=False)
+
+        # Write the YAML to a file
+        output_path.parent.mkdir(parents=True, exist_ok=True)
+        output_path.write_text('---\n' + header_yml + '---\n' + dict_to_md(body_parts,remove_keys=['Rubric','Solution','Comments','pl-submission-panel','pl-answer-panel']) +
+                        '\n## Attribution\n\n' + process_attribution(header.get('attribution')) )
+
+    # Move image assets
+    files_to_copy = header.get('assets')
+    if files_to_copy:
+        [copy2(pathlib.Path(source_filepath).parent / fl, output_path.parent) for fl in files_to_copy]
+
+def process_question_pl(source_filepath, output_path = None):
 
     try:
-        input_file = pathlib.Path(input_file)
+        pathlib.Path(source_filepath)
     except:
-        print("File does not exist.")
+        print(f"{source_filepath} - File does not exist.")
         raise
 
-    output_path = pathlib.Path(output_path)
+    if output_path is None:
+        output_path = pathlib.Path(source_filepath.replace('source','output/prairielearn')).parent
+    else:
+        output_path = pathlib.Path(output_path).parent
+
+        ## TODO: Make this a bit more robust
+        print(f"Warning: This feature (specifying your own directory {output_path}) is not tested!")
 
     # Parse the MD file
-    parsed_q = read_md_problem(input_file)
+    parsed_q = read_md_problem(source_filepath)
 
     # Create output dir if it doesn't exist
-    pathlib.Path(output_path).mkdir(parents=True, exist_ok=True)
+    output_path.mkdir(parents=True, exist_ok=True)
 
     ############### Start Sketchiest Part
     # Run the python code
     try:
-        exec(parsed_q['header']['server'].split('# Update the data object with a new dict')[0],globals())
+        exec(parsed_q['header']['server']['imports'],globals() )
+        exec(parsed_q['header']['server']['generate'].split('# Update the data object with a new dict')[0],globals() )
     except ModuleNotFoundError:
         # AWFUL AWFUL hack because of the prairielearn.py file
-        exec(parsed_q['header']['server'].replace('import prairielearn as pl','from . import prairielearn as pl').split('# Update the data object with a new dict')[0],globals())
+        exec(parsed_q['header']['server']['imports'].replace('import prairielearn as pl','from . import prairielearn as pl'),globals() )
+        exec(parsed_q['header']['server']['generate'].split('# Update the data object with a new dict')[0],globals() )     
     ############### End Sketchiest Part
 
     # Write info.json file
@@ -404,7 +548,7 @@ def process_question(input_file, output_path):
 
     # Question Preamble
     if parsed_q['body_parts']['preamble']:
-        question_html = f"{parsed_q['body_parts']['preamble']}\n\n"
+        question_html = f"<markdown>\n{ codecs.unicode_escape_decode(parsed_q['body_parts']['preamble'])[0] }\n</markdown>\n\n"
     else:
         question_html = f""
 
@@ -449,8 +593,16 @@ def process_question(input_file, output_path):
 
             question_html += "</div>\n</div>\n\n\n"
 
+    # Add pl-submission-panel and pl-answer-panel
+
+    if parsed_q['body_parts_split']['pl-submission-panel']:
+        question_html += f"<pl-submission-panel>{ parsed_q['body_parts_split']['pl-submission-panel'] } </pl-submission-panel>\n"
+
+    if parsed_q['body_parts_split']['pl-answer-panel']:
+        question_html += f"<pl-answer-panel>{ parsed_q['body_parts_split']['pl-answer-panel'] } </pl-answer-panel>\n"
+
     # Add Attribution
-    question_html += f"\n\n<markdown>---\n{process_attribution(parsed_q['header'].get('attribution'))}\n</markdown>"
+    question_html += f"\n<markdown>---\n{process_attribution(parsed_q['header'].get('attribution'))}\n</markdown>"
 
     # Final pre-processing
     question_html = pl_image_path(question_html)
@@ -470,21 +622,18 @@ def process_question(input_file, output_path):
     if files_to_copy:
         pl_path =  output_path / "clientFilesQuestion"
         pl_path.mkdir(parents=True, exist_ok=True)
-        [copy2(input_file.parent / fl, pl_path / fl) for fl in files_to_copy]
+        [copy2(pathlib.Path(source_filepath).parent / fl, pl_path / fl) for fl in files_to_copy]
 
 def pl_image_path(html):
 
-    """Adds `clientFilesQuestion` directory before the path automatically
+    """Adds `{{options.client_files_question_url}}` directory before the path automatically
     """
 
-    # If image files are included as markdown format, add clientFilesQuestion
-    res = re.subn("\((.*\.png)\)",'(clientFilesQuestion/\\1)',html)
-
-    # If image files are included as html format, add clientFilesQuestion
-    res = re.subn(r"src=\"(.*\.png)",
-              "src=\"clientFilesQuestion/\\1",res[0]) # works
-
-
-    return res[0]
+    # If image files are included as markdown format, add {{options.client_files_question_url}}
+    res = re.subn(r"\(((?!http).*\.png)\)",'({{options.client_files_question_url}}/\\1)',html)
 
+    # If image files are included as html format, add {{options.client_files_question_url}}
+    res = re.subn(r"src=\"(?!http)(.*\.png)",
+              "src=\"{{options.client_files_question_url}}/\\1",res[0]) # works
 
+    return res[0]
\ No newline at end of file
